﻿module NUnit.TickSpec

open NUnit.Framework
open System.Reflection
open TickSpec

/// Current assembly
let ass = Assembly.GetExecutingAssembly()
/// Available step definitions
let definitions = new StepDefinitions(ass)

/// Raises a not implemented exception
let inline pending () = 
    raise (new System.NotImplementedException()) |> ignore

/// Executes specified embedded feature file
let execute source =
    let names = ass.GetManifestResourceNames()
    let s = ass.GetManifestResourceStream(source)   
    definitions.Execute(source,s)

/// Inherit from FeatureFixture to define a feature fixture
[<AbstractClass>]
[<TestFixture>]
type FeatureFixture (source:string) =
    [<Test>]
    [<TestCaseSource("Scenarios")>]
    member this.TestScenario (scenario:Scenario) =
        if scenario.Tags |> Seq.exists ((=) "ignore") then
            raise (new IgnoreException("Ignored: " + scenario.Name))
        scenario.Action.Invoke()
    member this.Scenarios =       
        let s = ass.GetManifestResourceStream(source)   
        definitions.GenerateScenarios(source,s)